body {
	max-width: 80em;
	margin: 0;
	margin-left: auto;
	margin-right: auto;
	background: #FFF;
	color: #000;
}

#titlearea {
	display: none;
}

h1 {
	font-size: 180%;
	font-weight: 900;
}

h2 {
	font-size: 140%;
	font-weight: 700;
}

h3 {
	font-size: 120%;
	font-weight: 700;
}

h4 {
	font-size: 110%;
	font-weight: 700;
}

h5 {
	font-size: 100%;
	font-weight: 700;
}

h6 {
	font-size: 100%;
	font-weight: 600;
}

p {
	margin: 0 0 1em 0;
}

dt {
	font-weight: 700;
}

p.startli,p.startdd,p.starttd {
	margin-top: 2px;
}

p.endli {
	margin-bottom: 0;
}

p.enddd {
	margin-bottom: 4px;
}

p.endtd {
	margin-bottom: 2px;
}

caption {
	font-weight: 700;
}

span.legend {
	font-size: small;
	text-align: center;
}

h3.version {
	font-size: small;
	text-align: center;
}

div.qindex,div.navtab {
	background-color: #EBEFF6;
	border: 1px solid #A3B4D7;
	text-align: center;
	margin: 2px;
	padding: 2px;
}

div.navtab {
	margin-right: 15px;
}

/* @group Link Styling */
a {
	color: #546E00;
	text-decoration: none;
}

.contents a:visited {
	color: #344E00;
}

a:hover {
	text-decoration: underline;
}

a.qindexHL {
	background-color: #9CAFD4;
	color: #FFF;
	border: 1px double #869DCA;
}

code {
	color: #444;
}

a.code {
	color: #4665A2;
}

a.codeRef {
	color: #4665A2;
}

/* @end */
dl.el {
	margin-left: -1cm;
}

.fragment {
	font-family: monospace, fixed;
}

pre.fragment {
	border: 1px solid #C4C4C4;
	background-color: #F9F9F9;
	padding: 4px 6px;
	margin: 4px 8px 4px 2px;
	overflow: auto;
	line-height: 125%;
}

div.ah {
	background-color: #000;
	font-weight: 700;
	color: #FFF;
	margin-bottom: 3px;
	margin-top: 3px;
	padding: .2em;
	border: thin solid #333;
}

div.groupHeader {
	margin-left: 16px;
	margin-top: 12px;
	margin-bottom: 6px;
	font-weight: 700;
}

a + h2.groupheader {
	display: none;
}

div.groupText {
	margin-left: 16px;
	font-style: italic;
}

div.contents {
	margin-top: 10px;
	margin-left: 10px;
	margin-right: 10px;
}

td.indexkey {
	background-color: #EBEFF6;
	font-weight: 700;
	border: 1px solid #C4CFE5;
	margin: 2px 0;
	padding: 2px 10px;
}

td.indexvalue {
	background-color: #EBEFF6;
	border: 1px solid #C4CFE5;
	padding: 2px 10px;
	margin: 2px 0;
}

tr.memlist {
	background-color: #EEF1F7;
}

p.formulaDsp {
	text-align: center;
}

img.formulaInl {
	vertical-align: middle;
}

div.center {
	text-align: center;
	margin-top: 0;
	margin-bottom: 0;
	padding: 0;
}

div.center img {
	border: 0;
}

address.footer {
	text-align: right;
	padding-right: 12px;
}

img.footer {
	border: 0;
	vertical-align: middle;
}

/* @group Code Colorization */
span.keyword {
	color: green;
}

span.keywordtype {
	color: #3E873E;
}

span.keywordflow {
	color: #e08000;
}

span.comment {
	color: maroon;
}

span.preprocessor {
	color: #806020;
}

span.stringliteral {
	color: #002080;
}

span.charliteral {
	color: teal;
}

span.vhdldigit {
	color: #F0F;
}

span.vhdlkeyword {
	color: #700070;
}

span.vhdllogic {
	color: red;
}

/* @end */
td.tiny {
	font-size: x-small;
}

.dirtab {
	padding: 4px;
	border-collapse: collapse;
	border: 1px solid #A3B4D7;
}

th.dirtab {
	background: #EBEFF6;
	font-weight: 700;
}

hr {
	height: 0;
	border: none;
	border-top: 1px solid #DDD;
	margin: 2em 0 1em;
}

hr.footer {
	height: 1px;
}

/* @group Member Descriptions */
table.memberdecls {
	border-spacing: 0.125em;
}

h2.groupheader {
	margin: 0.5em 0 0.25em 0;
}

.mdescLeft,.mdescRight,.memItemLeft,.memItemRight,.memTemplItemLeft,.memTemplItemRight,.memTemplParams {
	margin: 0;
	padding: 0;
}

.mdescLeft,.mdescRight {
	color: #555;
}

.memItemLeft,.memItemRight,.memTemplParams {
	border: 0;
	font-family: monospace, fixed;
}

.memItemLeft,.memTemplItemLeft {
	white-space: nowrap;
	padding-left: 2em;
	padding-right: 1em;
}

.memItemLeft a.el {
	font-weight: bold;
}

.memTemplParams {
	color: #464646;
	white-space: nowrap;
}

td.memSeparator {
	display: none;
}

td.mlabels-right {
	vertical-align: top;
	padding-top: 4px;
	color: #AA6;
}

.memtitle {
	display: none;
}

/* @end */
/* @group Member Details */
/* Styles for detailed member documentation */
.memtemplate {
	color: #4665A2;
	font-weight: bold;
}

.memnav {
	background-color: #EBEFF6;
	border: 1px solid #A3B4D7;
	text-align: center;
	margin: 2px;
	margin-right: 15px;
	padding: 2px;
}

.memitem {
	padding: 0;
	margin: 1em 0 1em 0;
}

.memproto {
	padding: 0;
	font-weight: bold;
	color: #000;
}

.memproto .paramname {
	color: #444;
	font-style: normal;
}

.memdoc {
	padding: 0 0 0.5em 2em;
}

.paramkey {
	text-align: right;
}

.paramtype {
	color: #3E873E;
	white-space: nowrap;
}

.paramname {
	color: #444;
	white-space: nowrap;
	font-weight: bold;
}

td.paramname {
	vertical-align: top;
}

.fieldname {
	color: #000;
}

td.fieldname {
	padding-right: 1em;
	vertical-align: top;
}

td.fieldtype {
	vertical-align: top;
	color: #444;
	padding-right: 0.5em;
}

td.fielddoc p {
	margin: 0;
}

/* @end */
/* @group Directory (tree) */
/* for the tree view */
.ftvtree {
	font-family: sans-serif;
	margin: 0;
}

/* these are for tree view when used as main index */
.directory {
	font-size: small;
	margin: 0.5em;
}

.directory h3 {
	margin: 0;
	margin-top: 1em;
	font-size: 11pt;
}

.directory > h3 {
	margin-top: 0;
}

.directory p {
	margin: 0;
	white-space: nowrap;
}

.directory div {
	display: none;
	margin: 0;
}

.directory img {
	vertical-align: -30%;
}

/* these are for tree view when not used as main index */
.directory-alt {
	font-size: 100%;
	font-weight: bold;
}

.directory-alt h3 {
	margin: 0;
	margin-top: 1em;
	font-size: 11pt;
}

.directory-alt > h3 {
	margin-top: 0;
}

.directory-alt p {
	margin: 0;
	white-space: nowrap;
}

.directory-alt div {
	display: none;
	margin: 0;
}

.directory-alt img {
	vertical-align: -30%;
}

/* @end */
div.dynheader {
	margin-top: 8px;
}

address {
	font-style: normal;
	color: #2A3D61;
}

table.doxtable {
	border-collapse: collapse;
	margin: 0.5em;
}

table.doxtable td,table.doxtable th {
	border: 1px solid #DDD;
	padding: 3px 7px 2px;
}

table.doxtable th {
	background-color: #F3F3F3;
	color: #000;
	padding-bottom: 4px;
	padding-top: 5px;
	text-align: left;
	font-weight: bold;
}

.tabsearch {
	top: 0;
	left: 10px;
	height: 36px;
	z-index: 101;
	overflow: hidden;
	font-size: 13px;
}

div.navpath {
	padding: 0.25em;
}

.navpath ul {
	font-size: x-small;
	color: #8AA0CC;
	overflow: hidden;
	margin: 0;
	padding: 0;
}

.navpath li {
	list-style-type: none;
	float: left;
	padding-left: 10px;
	padding-right: 15px;
	color: #364D7C;
}

.navpath a {
	display: block;
	text-decoration: none;
	outline: none;
}

.navpath a:hover {
	color: #6884BD;
}

div.summary {
	float: right;
	font-size: x-small;
	padding: 0.25em 0.5em 0 0;
	width: 50%;
	text-align: right;
}

div.summary a {
	white-space: nowrap;
}

div.header {
	background-color: #F3F3F3;
	margin: 0;
	border: 0;
}

div.headertitle {
	font-size: 180%;
	font-weight: bold;
	color: #FFF;
	padding: 0.125em 0.25em 0.125em 0.25em;
	background-color: #333;
	background: linear-gradient(to bottom, #333 0%, #111 100%);
	border: solid 1px #444;
	border-top: 0;
	border-radius: 0 0 6px 6px;
}

div.line {
	font-family: monospace, fixed;
	font-size: 13px;
	min-height: 13px;
	line-height: 1.0;
	text-wrap: avoid;
	white-space: pre-wrap;
	text-indent: -53px;
	padding-left: 53px;
	padding-bottom: 0;
	margin: 0;
}

.glow {
	background-color: cyan;
	box-shadow: 0 0 10px cyan;
}

span.lineno {
	padding-right: 4px;
	text-align: right;
	border-right: 2px solid #0F0;
	background-color: #E8E8E8;
	white-space: pre;
}
span.lineno a {
	background-color: #D8D8D8;
}

span.lineno a:hover {
	background-color: #C8C8C8;
}

.tabs, .tabs2, .navpath {
	background-image: none;
	background-color: #333;
	background: linear-gradient(to bottom, #333 0%, #111 100%);
	border: 0;
	border-bottom: solid 2px #000;
	padding: 0;
	padding-top: 2px;
	font-size: small;
}

#navrow1 {
	border: 0;
}

th {
	text-align: left;
}

.mlabel {
	padding: 0.125em;
}

/* tabs*/

.tablist {
	margin: 0;
	padding: 0;
	display: table;
}

.tablist li {
	display: table-cell;
	line-height: 2em;
	list-style: none;
	background-color: #333;
	background: linear-gradient(to bottom, #444 0%, #222 100%);
	border: 1px solid #222;
	border-bottom: 0;
	border-radius: 6px 6px 0 0;
	color: #DDD;
}

.tablist a {
	display: block;
	padding: 0 20px;
	font-weight: bold;
	color: #859900;
	text-decoration: none;
	outline: none;
}

.header a {
	color: #859900;
}

.tabs3 .tablist a {
	padding: 0 10px;
}

.tablist a:hover {
	color: #fff;
	text-shadow: 0 1px 1px rgba(0, 0, 0, 1.0);
	text-decoration: none;
}

.tablist li.current a {
	color: #fff;
	text-shadow: 0 1px 1px rgba(0, 0, 0, 1.0);
}

span.icon {
	display: none;
}
